Package: motifcounter
Type: Package
Title: R package for analysing TFBSs in DNA sequences
Version: 1.35.0
Date: 2017
Author: Wolfgang Kopp [aut, cre]
Suggests: knitr, rmarkdown, testthat, MotifDb, seqLogo, prettydoc
Imports: Biostrings, methods
Depends: R(>= 3.0)
Maintainer: Wolfgang Kopp <wolfgang.kopp@mdc-berlin.de>
Description: 'motifcounter' provides motif matching, motif counting
        and motif enrichment functionality based on position
        frequency matrices.
        The main features of the packages include the utilization
        of higher-order background models and accounting
        for self-overlapping motif matches when determining motif enrichment. 
        The background model allows to capture dinucleotide 
        (or higher-order nucleotide) composition adequately
        which may reduced model biases and misleading results compared
        to using simple GC background models.
        When conducting a motif enrichment analysis 
        based on the motif match count, the package
        relies on a compound Poisson distribution or alternatively
        a combinatorial model. These distribution account for self-overlapping
        motif structures as exemplified by repeat-like or palindromic motifs,
        and allow to determine the p-value and fold-enrichment for 
        a set of observed motif matches.
License: GPL-2
biocViews: Transcription,MotifAnnotation,SequenceMatching,Software
RoxygenNote: 6.0.1
VignetteBuilder: knitr
NeedsCompilation: yes
Collate: 'background_wrapper.R' 'comppoiss_wrapper.R'
        'combinatorial_wrapper.R' 'score_wrapper.R' 'count_wrapper.R'
        'enrichmentTest.R' 'forground_wrapper.R' 'markovmodel.R'
        'motifcounter-package.R' 'observed_wrapper.R' 'option.R'
        'overlap.R' 'simulate_wrapper.R' 'wrapper.R'
git_url: https://git.bioconductor.org/packages/motifcounter
git_branch: devel
git_last_commit: 9ac52e2
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
Packaged: 2025-10-31 22:58:11 UTC; biocbuild
