% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/background_wrapper.R
\name{getCounts}
\alias{getCounts}
\title{Accessor to slot counts}
\usage{
getCounts(obj)
}
\arguments{
\item{obj}{A Background object}
}
\value{
counts slot
}
\description{
Accessor to slot counts
}
