% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlap.R
\name{getBeta3p}
\alias{getBeta3p}
\title{Accessor to slot beta3p}
\usage{
getBeta3p(obj)
}
\arguments{
\item{obj}{An Overlap object}
}
\value{
beta3p slot
}
\description{
Accessor to slot beta3p
}
