% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/background_wrapper.R
\name{getOrder}
\alias{getOrder}
\title{Accessor to slot order}
\usage{
getOrder(obj)
}
\arguments{
\item{obj}{A Background object}
}
\value{
order slot
}
\description{
Accessor to slot order
}
