% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/background_wrapper.R
\name{getTrans}
\alias{getTrans}
\title{Accessor to slot trans}
\usage{
getTrans(obj)
}
\arguments{
\item{obj}{A Background object}
}
\value{
trans slot
}
\description{
Accessor to slot trans
}
